<?php
/*
 * Fusio is an open source API management platform which helps to create innovative API solutions.
 * For the current version and information visit <https://www.fusio-project.org/>
 *
 * Copyright 2015-2023 Christoph Kappestein <christoph.kappestein@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

namespace Fusio\Impl\Installation;

use Fusio\Adapter;
use Fusio\Engine\Inflection\ClassName;
use Fusio\Impl\Action\Scheme;
use Fusio\Impl\Authorization;
use Fusio\Impl\Authorization\TokenGenerator;
use Fusio\Impl\Backend;
use Fusio\Impl\Connection\System as ConnectionSystem;
use Fusio\Impl\Consumer;
use Fusio\Impl\System;
use Fusio\Impl\Table;
use Fusio\Marketplace;
use Fusio\Model;
use Psr\Container\ContainerInterface;
use PSX\Api\Model\Passthru;
use PSX\Schema\TypeFactory;

/**
 * NewInstallation
 *
 * @author  Christoph Kappestein <christoph.kappestein@gmail.com>
 * @license http://www.apache.org/licenses/LICENSE-2.0
 * @link    https://www.fusio-project.org
 */
class NewInstallation
{
    public static function getData(?string $tenantId = null): DataBag
    {
        $password = \password_hash(TokenGenerator::generateUserPassword(), PASSWORD_DEFAULT);

        $bag = new DataBag();
        $bag->addCategory('default', tenantId: $tenantId);
        $bag->addCategory('backend', tenantId: $tenantId);
        $bag->addCategory('consumer', tenantId: $tenantId);
        $bag->addCategory('system', tenantId: $tenantId);
        $bag->addCategory('authorization', tenantId: $tenantId);
        $bag->addRole('default', 'Administrator', tenantId: $tenantId);
        $bag->addRole('default', 'Backend', tenantId: $tenantId);
        $bag->addRole('default', 'Consumer', tenantId: $tenantId);
        $bag->addUser('Administrator', 'Administrator', 'admin@localhost.com', $password, tenantId: $tenantId);
        $bag->addScope('backend', 'backend', 'Global access to the backend API', tenantId: $tenantId);
        $bag->addScope('consumer', 'consumer', 'Global access to the consumer API', tenantId: $tenantId);
        $bag->addScope('authorization', 'authorization', 'Authorization API endpoint', tenantId: $tenantId);
        $bag->addScope('default', 'default', 'Default scope', tenantId: $tenantId);
        $bag->addConfig('app_approval', Table\Config::FORM_BOOLEAN, 0, 'If true the status of a new app is PENDING so that an administrator has to manually activate the app', tenantId: $tenantId);
        $bag->addConfig('consumer_max_apps', Table\Config::FORM_NUMBER, 16, 'The max amount of apps a consumer can generate', tenantId: $tenantId);
        $bag->addConfig('consumer_max_tokens', Table\Config::FORM_NUMBER, 16, 'The max amount of tokens a consumer can generate', tenantId: $tenantId);
        $bag->addConfig('consumer_max_webhooks', Table\Config::FORM_NUMBER, 8, 'The max amount of webhooks a consumer can register', tenantId: $tenantId);
        $bag->addConfig('authorization_url', Table\Config::FORM_STRING, '', 'Url where the user can authorize for the OAuth2 flow', tenantId: $tenantId);
        $bag->addConfig('info_title', Table\Config::FORM_STRING, 'Fusio', 'The title of the application', tenantId: $tenantId);
        $bag->addConfig('info_description', Table\Config::FORM_STRING, '', 'A short description of the application. CommonMark syntax MAY be used for rich text representation', tenantId: $tenantId);
        $bag->addConfig('info_tos', Table\Config::FORM_STRING, '', 'A URL to the Terms of Service for the API. MUST be in the format of a URL', tenantId: $tenantId);
        $bag->addConfig('info_contact_name', Table\Config::FORM_STRING, '', 'The identifying name of the contact person/organization', tenantId: $tenantId);
        $bag->addConfig('info_contact_url', Table\Config::FORM_STRING, '', 'The URL pointing to the contact information. MUST be in the format of a URL', tenantId: $tenantId);
        $bag->addConfig('info_contact_email', Table\Config::FORM_STRING, '', 'The email address of the contact person/organization. MUST be in the format of an email address', tenantId: $tenantId);
        $bag->addConfig('info_license_name', Table\Config::FORM_STRING, '', 'The license name used for the API', tenantId: $tenantId);
        $bag->addConfig('info_license_url', Table\Config::FORM_STRING, '', 'A URL to the license used for the API. MUST be in the format of a URL', tenantId: $tenantId);
        $bag->addConfig('mail_register_subject', Table\Config::FORM_STRING, 'Fusio registration', 'Subject of the activation mail', tenantId: $tenantId);
        $bag->addConfig('mail_register_body', Table\Config::FORM_TEXT, 'Hello {name},' . "\n\n" . 'you have successful registered at Fusio.' . "\n" . 'To activate you account please visit the following link:' . "\n" . '{apps_url}/developer/register/activate/{token}', 'Body of the activation mail', tenantId: $tenantId);
        $bag->addConfig('mail_pw_reset_subject', Table\Config::FORM_STRING, 'Fusio password reset', 'Subject of the password reset mail', tenantId: $tenantId);
        $bag->addConfig('mail_pw_reset_body', Table\Config::FORM_TEXT, 'Hello {name},' . "\n\n" . 'you have requested to reset your password.' . "\n" . 'To set a new password please visit the following link:' . "\n" . '{apps_url}/developer/password/confirm/{token}' . "\n\n" . 'Please ignore this email if you have not requested a password reset.', 'Body of the password reset mail', tenantId: $tenantId);
        $bag->addConfig('mail_points_subject', Table\Config::FORM_STRING, 'Fusio points threshold reached', 'Subject of the points threshold mail', tenantId: $tenantId);
        $bag->addConfig('mail_points_body', Table\Config::FORM_TEXT, 'Hello {name},' . "\n\n" . 'your account has reached the configured threshold of {points} points.' . "\n" . 'If your account reaches 0 points your are not longer able to invoke specific endpoints.' . "\n" . 'To prevent this please go to the developer portal to purchase new points:' . "\n" . '{apps_url}/developer', 'Body of the points threshold mail', tenantId: $tenantId);
        $bag->addConfig('recaptcha_key', Table\Config::FORM_STRING, '', 'ReCaptcha key', tenantId: $tenantId);
        $bag->addConfig('recaptcha_secret', Table\Config::FORM_STRING, '', 'ReCaptcha secret', tenantId: $tenantId);
        $bag->addConfig('payment_stripe_secret', Table\Config::FORM_STRING, '', 'The stripe webhook secret which is needed to verify a webhook request', tenantId: $tenantId);
        $bag->addConfig('payment_stripe_portal_configuration', Table\Config::FORM_STRING, '', 'The stripe portal configuration id', tenantId: $tenantId);
        $bag->addConfig('payment_currency', Table\Config::FORM_STRING, '', 'The three-character ISO-4217 currency code which is used to process payments', tenantId: $tenantId);
        $bag->addConfig('role_default', Table\Config::FORM_STRING, 'Consumer', 'Default role which a user gets assigned on registration', tenantId: $tenantId);
        $bag->addConfig('points_default', Table\Config::FORM_NUMBER, 0, 'The default amount of points which a user receives if he registers', tenantId: $tenantId);
        $bag->addConfig('points_threshold', Table\Config::FORM_NUMBER, 0, 'If a user goes below this points threshold we send an information to the user', tenantId: $tenantId);
        $bag->addConfig('system_mailer', Table\Config::FORM_STRING, '', 'Optional the name of an SMTP connection which is used as mailer, by default the system uses the connection configured through the APP_MAILER environment variable', tenantId: $tenantId);
        $bag->addConfig('system_dispatcher', Table\Config::FORM_STRING, '', 'Optional the name of an HTTP or Message-Queue connection which is used to dispatch events. By default the system uses simply cron and an internal table to dispatch such events, for better performance you can provide a Message-Queue connection and Fusio will only dispatch the event to the queue, then your worker must execute the actual webhook HTTP request', tenantId: $tenantId);
        $bag->addConfig('user_pw_length', Table\Config::FORM_NUMBER, 8, 'Minimal required password length', tenantId: $tenantId);
        $bag->addConfig('user_approval', Table\Config::FORM_BOOLEAN, 1, 'Whether the user needs to activate the account through an email', tenantId: $tenantId);
        $bag->addConfig('marketplace_client_id', Table\Config::FORM_STRING, '', 'The marketplace client id', tenantId: $tenantId);
        $bag->addConfig('marketplace_client_secret', Table\Config::FORM_STRING, '', 'The marketplace client secret', tenantId: $tenantId);
        if ($tenantId === null) {
            // we add the system connection only at the root tenant
            $bag->addConnection('System', ClassName::serialize(ConnectionSystem::class), tenantId: $tenantId);
        }
        $bag->addRate('Default', 0, 3600, 'PT1H', tenantId: $tenantId);
        $bag->addRate('Default-Anonymous', 4, 900, 'PT1H', tenantId: $tenantId);
        $bag->addRateAllocation('Default', tenantId: $tenantId);
        $bag->addRateAllocation('Default-Anonymous', null, null, null, null, false, tenantId: $tenantId);
        $bag->addCronjob('backend', 'Renew_Token', '0 * * * *', Scheme::wrap(Backend\Action\Connection\RenewToken::class), tenantId: $tenantId);
        $bag->addRoleScope('Administrator', 'authorization', tenantId: $tenantId);
        $bag->addRoleScope('Administrator', 'backend', tenantId: $tenantId);
        $bag->addRoleScope('Administrator', 'consumer', tenantId: $tenantId);
        $bag->addRoleScope('Administrator', 'default', tenantId: $tenantId);
        $bag->addRoleScope('Backend', 'authorization', tenantId: $tenantId);
        $bag->addRoleScope('Backend', 'backend', tenantId: $tenantId);
        $bag->addRoleScope('Backend', 'default', tenantId: $tenantId);
        $bag->addRoleScope('Consumer', 'authorization', tenantId: $tenantId);
        $bag->addRoleScope('Consumer', 'consumer', tenantId: $tenantId);
        $bag->addRoleScope('Consumer', 'default', tenantId: $tenantId);
        $bag->addSchema('default', 'Passthru', Passthru::class, tenantId: $tenantId);
        $bag->addSchema('default', 'Message', Model\Common\Message::class, tenantId: $tenantId);
        $bag->addUserScope('Administrator', 'backend', tenantId: $tenantId);
        $bag->addUserScope('Administrator', 'consumer', tenantId: $tenantId);
        $bag->addUserScope('Administrator', 'authorization', tenantId: $tenantId);
        $bag->addUserScope('Administrator', 'default', tenantId: $tenantId);
        $bag->addPage('Overview', 'overview', self::readFile('overview.html'), Table\Page::STATUS_INVISIBLE, tenantId: $tenantId);
        $bag->addPage('Getting started', 'getting-started', self::readFile('getting-started.html'), tenantId: $tenantId);
        $bag->addPage('API', 'api', self::readFile('api.html'), tenantId: $tenantId);
        $bag->addPage('Authorization', 'authorization', self::readFile('authorization.html'), tenantId: $tenantId);
        $bag->addPage('Support', 'support', self::readFile('support.html'), tenantId: $tenantId);
        $bag->addPage('SDK', 'sdk', self::readFile('sdk.html'), tenantId: $tenantId);

        foreach (self::getOperations() as $category => $operations) {
            $bag->addOperations($tenantId, $category, $operations);
        }

        return $bag;
    }

    private static function getOperations(): array
    {
        return [
            'default' => [
                'meta.getAbout' => new Operation(
                    action: System\Action\Meta\GetAbout::class,
                    httpMethod: 'GET',
                    httpPath: '/',
                    httpCode: 200,
                    outgoing: Model\System\About::class,
                    public: true
                ),
            ],
            'backend' => [
                'account.get' => new Operation(
                    action: Backend\Action\Account\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/account',
                    httpCode: 200,
                    outgoing: Model\Backend\User::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'account.update' => new Operation(
                    action: Backend\Action\Account\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/account',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\UserUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'account.changePassword' => new Operation(
                    action: Backend\Action\Account\ChangePassword::class,
                    httpMethod: 'PUT',
                    httpPath: '/account/change_password',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\AccountChangePassword::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'action.getAll' => new Operation(
                    action: Backend\Action\Action\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/action',
                    httpCode: 200,
                    outgoing: Model\Backend\ActionCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'action.create' => new Operation(
                    action: Backend\Action\Action\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/action',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\ActionCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.action.create',
                ),
                'action.getClasses' => new Operation(
                    action: Backend\Action\Action\GetIndex::class,
                    httpMethod: 'GET',
                    httpPath: '/action/list',
                    httpCode: 200,
                    outgoing: Model\Backend\ActionIndex::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'action.getForm' => new Operation(
                    action: Backend\Action\Action\GetForm::class,
                    httpMethod: 'GET',
                    httpPath: '/action/form',
                    httpCode: 200,
                    outgoing: Model\Common\FormContainer::class,
                    parameters: ['class' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'action.execute' => new Operation(
                    action: Backend\Action\Action\Execute::class,
                    httpMethod: 'POST',
                    httpPath: '/action/execute/:action_id',
                    httpCode: 200,
                    outgoing: Model\Backend\ActionExecuteResponse::class,
                    incoming: Model\Backend\ActionExecuteRequest::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'action.get' => new Operation(
                    action: Backend\Action\Action\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/action/$action_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Action::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'action.update' => new Operation(
                    action: Backend\Action\Action\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/action/$action_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\ActionUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.action.update',
                ),
                'action.delete' => new Operation(
                    action: Backend\Action\Action\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/action/$action_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [404 => Model\Common\Message::class, 401 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.action.delete',
                ),
                'app.getAll' => new Operation(
                    action: Backend\Action\App\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/app',
                    httpCode: 200,
                    outgoing: Model\Backend\AppCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'app.create' => new Operation(
                    action: Backend\Action\App\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/app',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\AppCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.app.create',
                ),
                'app.get' => new Operation(
                    action: Backend\Action\App\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/app/$app_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\App::class,
                    throws: [404 => Model\Common\Message::class, 401 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'app.update' => new Operation(
                    action: Backend\Action\App\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/app/$app_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\AppUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.app.update',
                ),
                'app.delete' => new Operation(
                    action: Backend\Action\App\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/app/$app_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.app.delete',
                ),
                'app.deleteToken' => new Operation(
                    action: Backend\Action\App\DeleteToken::class,
                    httpMethod: 'DELETE',
                    httpPath: '/app/$app_id<[0-9]+>/token/:token_id',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'audit.getAll' => new Operation(
                    action: Backend\Action\Audit\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/audit',
                    httpCode: 200,
                    outgoing: Model\Backend\AuditCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'event' => TypeFactory::getString(), 'ip' => TypeFactory::getString(), 'message' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'audit.get' => new Operation(
                    action: Backend\Action\Audit\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/audit/$audit_id<[0-9]+>',
                    httpCode: 200,
                    outgoing: Model\Backend\Audit::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'category.getAll' => new Operation(
                    action: Backend\Action\Category\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/category',
                    httpCode: 200,
                    outgoing: Model\Backend\CategoryCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'category.create' => new Operation(
                    action: Backend\Action\Category\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/category',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\CategoryCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.category.create',
                ),
                'category.get' => new Operation(
                    action: Backend\Action\Category\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/category/$category_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Category::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'category.update' => new Operation(
                    action: Backend\Action\Category\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/category/$category_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\CategoryUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.category.update',
                ),
                'category.delete' => new Operation(
                    action: Backend\Action\Category\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/category/$category_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.category.delete',
                ),
                'config.getAll' => new Operation(
                    action: Backend\Action\Config\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/config',
                    httpCode: 200,
                    outgoing: Model\Backend\ConfigCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'config.get' => new Operation(
                    action: Backend\Action\Config\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/config/$config_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Config::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'config.update' => new Operation(
                    action: Backend\Action\Config\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/config/$config_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\ConfigUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.config.update',
                ),
                'connection.getAll' => new Operation(
                    action: Backend\Action\Connection\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/connection',
                    httpCode: 200,
                    outgoing: Model\Backend\ConnectionCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'connection.create' => new Operation(
                    action: Backend\Action\Connection\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/connection',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\ConnectionCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.connection.create',
                ),
                'connection.getClasses' => new Operation(
                    action: Backend\Action\Connection\GetIndex::class,
                    httpMethod: 'GET',
                    httpPath: '/connection/list',
                    httpCode: 200,
                    outgoing: Model\Backend\ConnectionIndex::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'connection.getForm' => new Operation(
                    action: Backend\Action\Connection\GetForm::class,
                    httpMethod: 'GET',
                    httpPath: '/connection/form',
                    httpCode: 200,
                    outgoing: Model\Common\FormContainer::class,
                    parameters: ['class' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'connection.get' => new Operation(
                    action: Backend\Action\Connection\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/connection/$connection_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Connection::class,
                    throws: [404 => Model\Common\Message::class, 401 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'connection.update' => new Operation(
                    action: Backend\Action\Connection\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/connection/$connection_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\ConnectionUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.connection.update',
                ),
                'connection.delete' => new Operation(
                    action: Backend\Action\Connection\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/connection/$connection_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.connection.delete',
                ),
                'connection.getRedirect' => new Operation(
                    action: Backend\Action\Connection\GetRedirect::class,
                    httpMethod: 'GET',
                    httpPath: '/connection/$connection_id<[0-9]+|^~>/redirect',
                    httpCode: 200,
                    outgoing: Model\Backend\ConnectionRedirectResponse::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'connection.getIntrospection' => new Operation(
                    action: Backend\Action\Connection\Introspection\GetEntities::class,
                    httpMethod: 'GET',
                    httpPath: '/connection/$connection_id<[0-9]+|^~>/introspection',
                    httpCode: 200,
                    outgoing: Model\Backend\ConnectionIntrospectionEntities::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'connection.getIntrospectionForEntity' => new Operation(
                    action: Backend\Action\Connection\Introspection\GetEntity::class,
                    httpMethod: 'GET',
                    httpPath: '/connection/$connection_id<[0-9]+|^~>/introspection/:entity',
                    httpCode: 200,
                    outgoing: Model\Backend\ConnectionIntrospectionEntity::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'cronjob.getAll' => new Operation(
                    action: Backend\Action\Cronjob\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/cronjob',
                    httpCode: 200,
                    outgoing: Model\Backend\CronjobCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'cronjob.create' => new Operation(
                    action: Backend\Action\Cronjob\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/cronjob',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\CronjobCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.cronjob.create',
                ),
                'cronjob.get' => new Operation(
                    action: Backend\Action\Cronjob\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/cronjob/$cronjob_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Cronjob::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'cronjob.update' => new Operation(
                    action: Backend\Action\Cronjob\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/cronjob/$cronjob_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\CronjobUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.cronjob.update',
                ),
                'cronjob.delete' => new Operation(
                    action: Backend\Action\Cronjob\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/cronjob/$cronjob_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.cronjob.delete',
                ),
                'dashboard.getAll' => new Operation(
                    action: Backend\Action\Dashboard\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/dashboard',
                    httpCode: 200,
                    outgoing: Model\Backend\Dashboard::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'database.getConnections' => new Operation(
                    action: Backend\Action\Database\GetConnections::class,
                    httpMethod: 'GET',
                    httpPath: '/database',
                    httpCode: 200,
                    outgoing: Model\Backend\DatabaseConnections::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'database.getTables' => new Operation(
                    action: Backend\Action\Database\Table\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/database/:connection_id',
                    httpCode: 200,
                    outgoing: Model\Backend\DatabaseTables::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'database.getTable' => new Operation(
                    action: Backend\Action\Database\Table\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/database/:connection_id/:table_name',
                    httpCode: 200,
                    outgoing: Model\Backend\DatabaseTable::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'database.createTable' => new Operation(
                    action: Backend\Action\Database\Table\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/database/:connection_id',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\DatabaseTable::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'database.updateTable' => new Operation(
                    action: Backend\Action\Database\Table\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/database/:connection_id/:table_name',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\DatabaseTable::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'database.deleteTable' => new Operation(
                    action: Backend\Action\Database\Table\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/database/:connection_id/:table_name',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'database.getRows' => new Operation(
                    action: Backend\Action\Database\Row\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/database/:connection_id/:table_name/rows',
                    httpCode: 200,
                    outgoing: Model\Backend\DatabaseRows::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'filterBy' => TypeFactory::getString(), 'filterOp' => TypeFactory::getString(), 'filterValue' => TypeFactory::getString(), 'sortBy' => TypeFactory::getString(), 'sortOrder' => TypeFactory::getString(), 'columns' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'database.getRow' => new Operation(
                    action: Backend\Action\Database\Row\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/database/:connection_id/:table_name/rows/:id',
                    httpCode: 200,
                    outgoing: Model\Backend\DatabaseRow::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'database.createRow' => new Operation(
                    action: Backend\Action\Database\Row\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/database/:connection_id/:table_name/rows',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\DatabaseRow::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'database.updateRow' => new Operation(
                    action: Backend\Action\Database\Row\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/database/:connection_id/:table_name/rows/:id',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\DatabaseRow::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'database.deleteRow' => new Operation(
                    action: Backend\Action\Database\Row\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/database/:connection_id/:table_name/rows/:id',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'event.getAll' => new Operation(
                    action: Backend\Action\Event\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/event',
                    httpCode: 200,
                    outgoing: Model\Backend\EventCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'event.create' => new Operation(
                    action: Backend\Action\Event\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/event',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\EventCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.event.create',
                ),
                'event.get' => new Operation(
                    action: Backend\Action\Event\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/event/$event_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Event::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'event.update' => new Operation(
                    action: Backend\Action\Event\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/event/$event_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\EventUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.event.update',
                ),
                'event.delete' => new Operation(
                    action: Backend\Action\Event\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/event/$event_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.event.delete',
                ),
                'backup.export' => new Operation(
                    action: Backend\Action\Backup\Export::class,
                    httpMethod: 'POST',
                    httpPath: '/backup/export',
                    httpCode: 200,
                    outgoing: Model\Backend\BackupExport::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'backup.import' => new Operation(
                    action: Backend\Action\Backup\Import::class,
                    httpMethod: 'POST',
                    httpPath: '/backup/import',
                    httpCode: 200,
                    outgoing: Model\Backend\BackupImportResult::class,
                    incoming: Model\Backend\BackupImport::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'generator.getClasses' => new Operation(
                    action: Backend\Action\Generator\GetIndex::class,
                    httpMethod: 'GET',
                    httpPath: '/generator',
                    httpCode: 200,
                    outgoing: Model\Backend\GeneratorIndexProviders::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'generator.getForm' => new Operation(
                    action: Backend\Action\Generator\GetForm::class,
                    httpMethod: 'GET',
                    httpPath: '/generator/:provider',
                    httpCode: 200,
                    outgoing: Model\Common\FormContainer::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'generator.executeProvider' => new Operation(
                    action: Backend\Action\Generator\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/generator/:provider',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\GeneratorProvider::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'generator.getChangelog' => new Operation(
                    action: Backend\Action\Generator\Changelog::class,
                    httpMethod: 'PUT',
                    httpPath: '/generator/:provider',
                    httpCode: 200,
                    outgoing: Model\Backend\GeneratorProviderChangelog::class,
                    incoming: Model\Backend\GeneratorProviderConfig::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'identity.getAll' => new Operation(
                    action: Backend\Action\Identity\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/identity',
                    httpCode: 200,
                    outgoing: Model\Backend\IdentityCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'identity.create' => new Operation(
                    action: Backend\Action\Identity\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/identity',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\IdentityCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.identity.create',
                ),
                'identity.getClasses' => new Operation(
                    action: Backend\Action\Identity\GetIndex::class,
                    httpMethod: 'GET',
                    httpPath: '/identity/list',
                    httpCode: 200,
                    outgoing: Model\Backend\IdentityIndex::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'identity.getForm' => new Operation(
                    action: Backend\Action\Identity\GetForm::class,
                    httpMethod: 'GET',
                    httpPath: '/identity/form',
                    httpCode: 200,
                    outgoing: Model\Common\FormContainer::class,
                    parameters: ['class' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'identity.get' => new Operation(
                    action: Backend\Action\Identity\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/identity/$identity_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Identity::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'identity.update' => new Operation(
                    action: Backend\Action\Identity\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/identity/$identity_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\IdentityUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.identity.update',
                ),
                'identity.delete' => new Operation(
                    action: Backend\Action\Identity\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/identity/$identity_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.identity.delete',
                ),
                'log.getAllErrors' => new Operation(
                    action: Backend\Action\Log\Error\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/log/error',
                    httpCode: 200,
                    outgoing: Model\Backend\LogErrorCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'log.getError' => new Operation(
                    action: Backend\Action\Log\Error\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/log/error/$error_id<[0-9]+>',
                    httpCode: 200,
                    outgoing: Model\Backend\LogError::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'log.getAll' => new Operation(
                    action: Backend\Action\Log\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/log',
                    httpCode: 200,
                    outgoing: Model\Backend\LogCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'routeId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'ip' => TypeFactory::getString(), 'userAgent' => TypeFactory::getString(), 'method' => TypeFactory::getString(), 'path' => TypeFactory::getString(), 'header' => TypeFactory::getString(), 'body' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'log.get' => new Operation(
                    action: Backend\Action\Log\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/log/$log_id<[0-9]+>',
                    httpCode: 200,
                    outgoing: Model\Backend\Log::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'marketplace.action.getAll' => new Operation(
                    action: Backend\Action\Marketplace\Action\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/marketplace/action',
                    httpCode: 200,
                    outgoing: Marketplace\MarketplaceActionCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'query' => TypeFactory::getString()],
                    throws: [401 => Marketplace\MarketplaceMessage::class, 500 => Marketplace\MarketplaceMessage::class],
                ),
                'marketplace.action.install' => new Operation(
                    action: Backend\Action\Marketplace\Action\Install::class,
                    httpMethod: 'POST',
                    httpPath: '/marketplace/action',
                    httpCode: 201,
                    outgoing: Marketplace\MarketplaceMessage::class,
                    incoming: Marketplace\MarketplaceInstall::class,
                    throws: [400 => Marketplace\MarketplaceMessage::class, 401 => Marketplace\MarketplaceMessage::class, 500 => Marketplace\MarketplaceMessage::class],
                ),
                'marketplace.action.get' => new Operation(
                    action: Backend\Action\Marketplace\Action\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/marketplace/action/:user/:name',
                    httpCode: 200,
                    outgoing: Marketplace\MarketplaceAction::class,
                    throws: [401 => Marketplace\MarketplaceMessage::class, 404 => Marketplace\MarketplaceMessage::class, 410 => Marketplace\MarketplaceMessage::class, 500 => Marketplace\MarketplaceMessage::class],
                ),
                'marketplace.action.upgrade' => new Operation(
                    action: Backend\Action\Marketplace\Action\Upgrade::class,
                    httpMethod: 'PUT',
                    httpPath: '/marketplace/action/:user/:name',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'marketplace.app.getAll' => new Operation(
                    action: Backend\Action\Marketplace\App\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/marketplace/app',
                    httpCode: 200,
                    outgoing: Marketplace\MarketplaceAppCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'query' => TypeFactory::getString()],
                    throws: [401 => Marketplace\MarketplaceMessage::class, 500 => Marketplace\MarketplaceMessage::class],
                ),
                'marketplace.app.install' => new Operation(
                    action: Backend\Action\Marketplace\App\Install::class,
                    httpMethod: 'POST',
                    httpPath: '/marketplace/app',
                    httpCode: 201,
                    outgoing: Marketplace\MarketplaceMessage::class,
                    incoming: Marketplace\MarketplaceInstall::class,
                    throws: [400 => Marketplace\MarketplaceMessage::class, 401 => Marketplace\MarketplaceMessage::class, 500 => Marketplace\MarketplaceMessage::class],
                ),
                'marketplace.app.get' => new Operation(
                    action: Backend\Action\Marketplace\App\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/marketplace/app/:user/:name',
                    httpCode: 200,
                    outgoing: Marketplace\MarketplaceApp::class,
                    throws: [401 => Marketplace\MarketplaceMessage::class, 404 => Marketplace\MarketplaceMessage::class, 410 => Marketplace\MarketplaceMessage::class, 500 => Marketplace\MarketplaceMessage::class],
                ),
                'marketplace.app.upgrade' => new Operation(
                    action: Backend\Action\Marketplace\App\Upgrade::class,
                    httpMethod: 'PUT',
                    httpPath: '/marketplace/app/:user/:name',
                    httpCode: 200,
                    outgoing: Marketplace\MarketplaceMessage::class,
                    throws: [400 => Marketplace\MarketplaceMessage::class, 401 => Marketplace\MarketplaceMessage::class, 404 => Marketplace\MarketplaceMessage::class, 410 => Marketplace\MarketplaceMessage::class, 500 => Marketplace\MarketplaceMessage::class],
                ),
                'page.getAll' => new Operation(
                    action: Backend\Action\Page\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/page',
                    httpCode: 200,
                    outgoing: Model\Backend\PageCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'page.create' => new Operation(
                    action: Backend\Action\Page\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/page',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\PageCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.page.create',
                ),
                'page.get' => new Operation(
                    action: Backend\Action\Page\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/page/$page_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Page::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'page.update' => new Operation(
                    action: Backend\Action\Page\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/page/$page_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\PageUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.page.update',
                ),
                'page.delete' => new Operation(
                    action: Backend\Action\Page\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/page/$page_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.page.delete',
                ),
                'plan.getAll' => new Operation(
                    action: Backend\Action\Plan\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/plan',
                    httpCode: 200,
                    outgoing: Model\Backend\PlanCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'plan.create' => new Operation(
                    action: Backend\Action\Plan\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/plan',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\PlanCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.plan.create',
                ),
                'plan.get' => new Operation(
                    action: Backend\Action\Plan\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/plan/$plan_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Plan::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'plan.update' => new Operation(
                    action: Backend\Action\Plan\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/plan/$plan_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\PlanUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.plan.update',
                ),
                'plan.delete' => new Operation(
                    action: Backend\Action\Plan\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/plan/$plan_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.plan.delete',
                ),
                'rate.getAll' => new Operation(
                    action: Backend\Action\Rate\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/rate',
                    httpCode: 200,
                    outgoing: Model\Backend\RateCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'rate.create' => new Operation(
                    action: Backend\Action\Rate\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/rate',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\RateCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.rate.create',
                ),
                'rate.get' => new Operation(
                    action: Backend\Action\Rate\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/rate/$rate_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Rate::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'rate.update' => new Operation(
                    action: Backend\Action\Rate\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/rate/$rate_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\RateUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.rate.update',
                ),
                'rate.delete' => new Operation(
                    action: Backend\Action\Rate\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/rate/$rate_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.rate.delete',
                ),
                'role.getAll' => new Operation(
                    action: Backend\Action\Role\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/role',
                    httpCode: 200,
                    outgoing: Model\Backend\RoleCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'role.create' => new Operation(
                    action: Backend\Action\Role\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/role',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\RoleCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.role.create',
                ),
                'role.get' => new Operation(
                    action: Backend\Action\Role\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/role/$role_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Role::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'role.update' => new Operation(
                    action: Backend\Action\Role\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/role/$role_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\RoleUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.role.update',
                ),
                'role.delete' => new Operation(
                    action: Backend\Action\Role\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/role/$role_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.role.delete',
                ),
                'operation.getAll' => new Operation(
                    action: Backend\Action\Operation\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/operation',
                    httpCode: 200,
                    outgoing: Model\Backend\OperationCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'operation.create' => new Operation(
                    action: Backend\Action\Operation\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/operation',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\OperationCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.operation.create',
                ),
                'operation.get' => new Operation(
                    action: Backend\Action\Operation\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/operation/$operation_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Operation::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'operation.update' => new Operation(
                    action: Backend\Action\Operation\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/operation/$operation_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\OperationUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.operation.update',
                ),
                'operation.delete' => new Operation(
                    action: Backend\Action\Operation\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/operation/$operation_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.operation.delete',
                ),
                'schema.getAll' => new Operation(
                    action: Backend\Action\Schema\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/schema',
                    httpCode: 200,
                    outgoing: Model\Backend\SchemaCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'schema.create' => new Operation(
                    action: Backend\Action\Schema\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/schema',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\SchemaCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.schema.create',
                ),
                'schema.getPreview' => new Operation(
                    action: Backend\Action\Schema\GetPreview::class,
                    httpMethod: 'POST',
                    httpPath: '/schema/preview/:schema_id',
                    httpCode: 200,
                    outgoing: Model\Backend\SchemaPreviewResponse::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'schema.updateForm' => new Operation(
                    action: Backend\Action\Schema\Form::class,
                    httpMethod: 'PUT',
                    httpPath: '/schema/form/$schema_id<[0-9]+>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\SchemaForm::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'schema.get' => new Operation(
                    action: Backend\Action\Schema\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/schema/$schema_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Schema::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'schema.update' => new Operation(
                    action: Backend\Action\Schema\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/schema/$schema_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\SchemaUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.schema.update',
                ),
                'schema.delete' => new Operation(
                    action: Backend\Action\Schema\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/schema/$schema_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.schema.delete',
                ),
                'scope.getAll' => new Operation(
                    action: Backend\Action\Scope\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/scope',
                    httpCode: 200,
                    outgoing: Model\Backend\ScopeCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'scope.create' => new Operation(
                    action: Backend\Action\Scope\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/scope',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\ScopeCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.scope.create',
                ),
                'scope.getCategories' => new Operation(
                    action: Backend\Action\Scope\GetCategories::class,
                    httpMethod: 'GET',
                    httpPath: '/scope/categories',
                    httpCode: 200,
                    outgoing: Model\Backend\ScopeCategories::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'scope.get' => new Operation(
                    action: Backend\Action\Scope\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/scope/$scope_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Scope::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'scope.update' => new Operation(
                    action: Backend\Action\Scope\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/scope/$scope_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\ScopeUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.scope.update',
                ),
                'scope.delete' => new Operation(
                    action: Backend\Action\Scope\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/scope/$scope_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.scope.delete',
                ),
                'sdk.getAll' => new Operation(
                    action: Backend\Action\Sdk\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/sdk',
                    httpCode: 200,
                    outgoing: Model\Backend\SdkResponse::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'sdk.generate' => new Operation(
                    action: Backend\Action\Sdk\Generate::class,
                    httpMethod: 'POST',
                    httpPath: '/sdk',
                    httpCode: 200,
                    outgoing: Model\Backend\SdkMessage::class,
                    incoming: Model\Backend\SdkGenerate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'statistic.getActivitiesPerUser' => new Operation(
                    action: Backend\Action\Statistic\GetActivitiesPerUser::class,
                    httpMethod: 'GET',
                    httpPath: '/statistic/activities_per_user',
                    httpCode: 200,
                    outgoing: Model\Backend\StatisticChart::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'operationId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'ip' => TypeFactory::getString(), 'userAgent' => TypeFactory::getString(), 'method' => TypeFactory::getString(), 'path' => TypeFactory::getString(), 'header' => TypeFactory::getString(), 'body' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'statistic.getCountRequests' => new Operation(
                    action: Backend\Action\Statistic\GetCountRequests::class,
                    httpMethod: 'GET',
                    httpPath: '/statistic/count_requests',
                    httpCode: 200,
                    outgoing: Model\Backend\StatisticCount::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'operationId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'ip' => TypeFactory::getString(), 'userAgent' => TypeFactory::getString(), 'method' => TypeFactory::getString(), 'path' => TypeFactory::getString(), 'header' => TypeFactory::getString(), 'body' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'statistic.getErrorsPerOperation' => new Operation(
                    action: Backend\Action\Statistic\GetErrorsPerOperation::class,
                    httpMethod: 'GET',
                    httpPath: '/statistic/errors_per_operation',
                    httpCode: 200,
                    outgoing: Model\Backend\StatisticChart::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'operationId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'ip' => TypeFactory::getString(), 'userAgent' => TypeFactory::getString(), 'method' => TypeFactory::getString(), 'path' => TypeFactory::getString(), 'header' => TypeFactory::getString(), 'body' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'statistic.getIncomingRequests' => new Operation(
                    action: Backend\Action\Statistic\GetIncomingRequests::class,
                    httpMethod: 'GET',
                    httpPath: '/statistic/incoming_requests',
                    httpCode: 200,
                    outgoing: Model\Backend\StatisticChart::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'operationId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'ip' => TypeFactory::getString(), 'userAgent' => TypeFactory::getString(), 'method' => TypeFactory::getString(), 'path' => TypeFactory::getString(), 'header' => TypeFactory::getString(), 'body' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'statistic.getIncomingTransactions' => new Operation(
                    action: Backend\Action\Statistic\GetIncomingTransactions::class,
                    httpMethod: 'GET',
                    httpPath: '/statistic/incoming_transactions',
                    httpCode: 200,
                    outgoing: Model\Backend\StatisticChart::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'operationId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'ip' => TypeFactory::getString(), 'userAgent' => TypeFactory::getString(), 'method' => TypeFactory::getString(), 'path' => TypeFactory::getString(), 'header' => TypeFactory::getString(), 'body' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'statistic.getIssuedTokens' => new Operation(
                    action: Backend\Action\Statistic\GetIssuedTokens::class,
                    httpMethod: 'GET',
                    httpPath: '/statistic/issued_tokens',
                    httpCode: 200,
                    outgoing: Model\Backend\StatisticChart::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'operationId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'ip' => TypeFactory::getString(), 'userAgent' => TypeFactory::getString(), 'method' => TypeFactory::getString(), 'path' => TypeFactory::getString(), 'header' => TypeFactory::getString(), 'body' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'statistic.getMostUsedActivities' => new Operation(
                    action: Backend\Action\Statistic\GetMostUsedActivities::class,
                    httpMethod: 'GET',
                    httpPath: '/statistic/most_used_activities',
                    httpCode: 200,
                    outgoing: Model\Backend\StatisticChart::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'operationId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'ip' => TypeFactory::getString(), 'userAgent' => TypeFactory::getString(), 'method' => TypeFactory::getString(), 'path' => TypeFactory::getString(), 'header' => TypeFactory::getString(), 'body' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'statistic.getMostUsedApps' => new Operation(
                    action: Backend\Action\Statistic\GetMostUsedApps::class,
                    httpMethod: 'GET',
                    httpPath: '/statistic/most_used_apps',
                    httpCode: 200,
                    outgoing: Model\Backend\StatisticChart::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'operationId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'ip' => TypeFactory::getString(), 'userAgent' => TypeFactory::getString(), 'method' => TypeFactory::getString(), 'path' => TypeFactory::getString(), 'header' => TypeFactory::getString(), 'body' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'statistic.getMostUsedOperations' => new Operation(
                    action: Backend\Action\Statistic\GetMostUsedOperations::class,
                    httpMethod: 'GET',
                    httpPath: '/statistic/most_used_operations',
                    httpCode: 200,
                    outgoing: Model\Backend\StatisticChart::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'operationId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'ip' => TypeFactory::getString(), 'userAgent' => TypeFactory::getString(), 'method' => TypeFactory::getString(), 'path' => TypeFactory::getString(), 'header' => TypeFactory::getString(), 'body' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'statistic.getTestCoverage' => new Operation(
                    action: Backend\Action\Statistic\GetTestCoverage::class,
                    httpMethod: 'GET',
                    httpPath: '/statistic/test_coverage',
                    httpCode: 200,
                    outgoing: Model\Backend\StatisticChart::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'statistic.getTimeAverage' => new Operation(
                    action: Backend\Action\Statistic\GetTimeAverage::class,
                    httpMethod: 'GET',
                    httpPath: '/statistic/time_average',
                    httpCode: 200,
                    outgoing: Model\Backend\StatisticChart::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'operationId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'ip' => TypeFactory::getString(), 'userAgent' => TypeFactory::getString(), 'method' => TypeFactory::getString(), 'path' => TypeFactory::getString(), 'header' => TypeFactory::getString(), 'body' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'statistic.getTimePerOperation' => new Operation(
                    action: Backend\Action\Statistic\GetTimePerOperation::class,
                    httpMethod: 'GET',
                    httpPath: '/statistic/time_per_operation',
                    httpCode: 200,
                    outgoing: Model\Backend\StatisticChart::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'operationId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'ip' => TypeFactory::getString(), 'userAgent' => TypeFactory::getString(), 'method' => TypeFactory::getString(), 'path' => TypeFactory::getString(), 'header' => TypeFactory::getString(), 'body' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'statistic.getUsedPoints' => new Operation(
                    action: Backend\Action\Statistic\GetUsedPoints::class,
                    httpMethod: 'GET',
                    httpPath: '/statistic/used_points',
                    httpCode: 200,
                    outgoing: Model\Backend\StatisticChart::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'operationId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'ip' => TypeFactory::getString(), 'userAgent' => TypeFactory::getString(), 'method' => TypeFactory::getString(), 'path' => TypeFactory::getString(), 'header' => TypeFactory::getString(), 'body' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'statistic.getUserRegistrations' => new Operation(
                    action: Backend\Action\Statistic\GetUserRegistrations::class,
                    httpMethod: 'GET',
                    httpPath: '/statistic/user_registrations',
                    httpCode: 200,
                    outgoing: Model\Backend\StatisticChart::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'operationId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'ip' => TypeFactory::getString(), 'userAgent' => TypeFactory::getString(), 'method' => TypeFactory::getString(), 'path' => TypeFactory::getString(), 'header' => TypeFactory::getString(), 'body' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'tenant.setup' => new Operation(
                    action: Backend\Action\Tenant\Setup::class,
                    httpMethod: 'PUT',
                    httpPath: '/tenant/:tenant_id',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'tenant.remove' => new Operation(
                    action: Backend\Action\Tenant\Remove::class,
                    httpMethod: 'DELETE',
                    httpPath: '/tenant/:tenant_id',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'test.getAll' => new Operation(
                    action: Backend\Action\Test\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/test',
                    httpCode: 200,
                    outgoing: Model\Backend\TestCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'test.get' => new Operation(
                    action: Backend\Action\Test\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/test/$test_id<[0-9]+>',
                    httpCode: 200,
                    outgoing: Model\Backend\Test::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'test.refresh' => new Operation(
                    action: Backend\Action\Test\Refresh::class,
                    httpMethod: 'PUT',
                    httpPath: '/test',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'test.run' => new Operation(
                    action: Backend\Action\Test\Run::class,
                    httpMethod: 'POST',
                    httpPath: '/test',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'test.update' => new Operation(
                    action: Backend\Action\Test\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/test/$test_id<[0-9]+>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\Test::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'token.getAll' => new Operation(
                    action: Backend\Action\Token\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/token',
                    httpCode: 200,
                    outgoing: Model\Backend\TokenCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'appId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'status' => TypeFactory::getInteger(), 'scope' => TypeFactory::getString(), 'ip' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'token.get' => new Operation(
                    action: Backend\Action\Token\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/token/$token_id<[0-9]+>',
                    httpCode: 200,
                    outgoing: Model\Backend\Token::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'transaction.getAll' => new Operation(
                    action: Backend\Action\Transaction\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/transaction',
                    httpCode: 200,
                    outgoing: Model\Backend\TransactionCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString(), 'from' => TypeFactory::getDateTime(), 'to' => TypeFactory::getDateTime(), 'planId' => TypeFactory::getInteger(), 'userId' => TypeFactory::getInteger(), 'appId' => TypeFactory::getInteger(), 'status' => TypeFactory::getString(), 'provider' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'transaction.get' => new Operation(
                    action: Backend\Action\Transaction\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/transaction/$transaction_id<[0-9]+>',
                    httpCode: 200,
                    outgoing: Model\Backend\Transaction::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'trash.getTypes' => new Operation(
                    action: Backend\Action\Trash\GetTypes::class,
                    httpMethod: 'GET',
                    httpPath: '/trash',
                    httpCode: 200,
                    outgoing: Model\Backend\TrashTypes::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'trash.getAllByType' => new Operation(
                    action: Backend\Action\Trash\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/trash/:type',
                    httpCode: 200,
                    outgoing: Model\Backend\TrashDataCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'trash.restore' => new Operation(
                    action: Backend\Action\Trash\Restore::class,
                    httpMethod: 'POST',
                    httpPath: '/trash/:type',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\TrashRestore::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'user.getAll' => new Operation(
                    action: Backend\Action\User\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/user',
                    httpCode: 200,
                    outgoing: Model\Backend\UserCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'user.create' => new Operation(
                    action: Backend\Action\User\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/user',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\UserCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.user.create',
                ),
                'user.get' => new Operation(
                    action: Backend\Action\User\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/user/$user_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\User::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'user.update' => new Operation(
                    action: Backend\Action\User\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/user/$user_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\UserUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.user.update',
                ),
                'user.delete' => new Operation(
                    action: Backend\Action\User\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/user/$user_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.user.delete',
                ),
                'webhook.getAll' => new Operation(
                    action: Backend\Action\Webhook\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/webhook',
                    httpCode: 200,
                    outgoing: Model\Backend\WebhookCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'webhook.create' => new Operation(
                    action: Backend\Action\Webhook\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/webhook',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\WebhookCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.webhook.create',
                ),
                'webhook.get' => new Operation(
                    action: Backend\Action\Webhook\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/webhook/$webhook_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Backend\Webhook::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'webhook.update' => new Operation(
                    action: Backend\Action\Webhook\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/webhook/$webhook_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\WebhookUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.webhook.update',
                ),
                'webhook.delete' => new Operation(
                    action: Backend\Action\Webhook\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/webhook/$webhook_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    eventName: 'fusio.webhook.delete',
                ),
            ],
            'consumer' => [
                'app.getAll' => new Operation(
                    action: Consumer\Action\App\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/app',
                    httpCode: 200,
                    outgoing: Model\Consumer\AppCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'app.create' => new Operation(
                    action: Consumer\Action\App\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/app',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Consumer\AppCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'app.get' => new Operation(
                    action: Consumer\Action\App\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/app/$app_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Consumer\App::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'app.update' => new Operation(
                    action: Consumer\Action\App\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/app/$app_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Consumer\AppUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'app.delete' => new Operation(
                    action: Consumer\Action\App\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/app/$app_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'event.getAll' => new Operation(
                    action: Consumer\Action\Event\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/event',
                    httpCode: 200,
                    outgoing: Model\Consumer\EventCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'grant.getAll' => new Operation(
                    action: Consumer\Action\Grant\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/grant',
                    httpCode: 200,
                    outgoing: Model\Consumer\GrantCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'grant.delete' => new Operation(
                    action: Consumer\Action\Grant\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/grant/$grant_id<[0-9]+>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'log.getAll' => new Operation(
                    action: Consumer\Action\Log\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/log',
                    httpCode: 200,
                    outgoing: Model\Consumer\LogCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'log.get' => new Operation(
                    action: Consumer\Action\Log\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/log/$log_id<[0-9]+>',
                    httpCode: 200,
                    outgoing: Model\Consumer\Log::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'page.getAll' => new Operation(
                    action: Consumer\Action\Page\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/page',
                    httpCode: 200,
                    outgoing: Model\Consumer\PageCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    public: true,
                ),
                'page.get' => new Operation(
                    action: Consumer\Action\Page\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/page/:page_id',
                    httpCode: 200,
                    outgoing: Model\Consumer\Page::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    public: true,
                ),
                'payment.portal' => new Operation(
                    action: Consumer\Action\Payment\Portal::class,
                    httpMethod: 'POST',
                    httpPath: '/payment/:provider/portal',
                    httpCode: 200,
                    outgoing: Model\Consumer\PaymentPortalResponse::class,
                    incoming: Model\Consumer\PaymentPortalRequest::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'payment.checkout' => new Operation(
                    action: Consumer\Action\Payment\Checkout::class,
                    httpMethod: 'POST',
                    httpPath: '/payment/:provider/checkout',
                    httpCode: 200,
                    outgoing: Model\Consumer\PaymentCheckoutResponse::class,
                    incoming: Model\Consumer\PaymentCheckoutRequest::class,
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'plan.getAll' => new Operation(
                    action: Consumer\Action\Plan\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/plan',
                    httpCode: 200,
                    outgoing: Model\Consumer\PlanCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'plan.get' => new Operation(
                    action: Consumer\Action\Plan\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/plan/$plan_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Consumer\Plan::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'scope.getAll' => new Operation(
                    action: Consumer\Action\Scope\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/scope',
                    httpCode: 200,
                    outgoing: Model\Consumer\ScopeCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'token.getAll' => new Operation(
                    action: Consumer\Action\Token\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/token',
                    httpCode: 200,
                    outgoing: Model\Consumer\TokenCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'token.create' => new Operation(
                    action: Consumer\Action\Token\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/token',
                    httpCode: 201,
                    outgoing: Model\Consumer\TokenAccessToken::class,
                    incoming: Model\Consumer\TokenCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'token.get' => new Operation(
                    action: Consumer\Action\Token\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/token/$token_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Consumer\Token::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'token.update' => new Operation(
                    action: Consumer\Action\Token\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/token/$token_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Consumer\TokenAccessToken::class,
                    incoming: Model\Consumer\TokenUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'token.delete' => new Operation(
                    action: Consumer\Action\Token\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/token/$token_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'webhook.getAll' => new Operation(
                    action: Consumer\Action\Webhook\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/webhook',
                    httpCode: 200,
                    outgoing: Model\Consumer\WebhookCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'webhook.create' => new Operation(
                    action: Consumer\Action\Webhook\Create::class,
                    httpMethod: 'POST',
                    httpPath: '/webhook',
                    httpCode: 201,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Consumer\WebhookCreate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'webhook.get' => new Operation(
                    action: Consumer\Action\Webhook\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/webhook/$webhook_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Consumer\Webhook::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'webhook.update' => new Operation(
                    action: Consumer\Action\Webhook\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/webhook/$webhook_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Consumer\WebhookUpdate::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'webhook.delete' => new Operation(
                    action: Consumer\Action\Webhook\Delete::class,
                    httpMethod: 'DELETE',
                    httpPath: '/webhook/$webhook_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'transaction.getAll' => new Operation(
                    action: Consumer\Action\Transaction\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/transaction',
                    httpCode: 200,
                    outgoing: Model\Consumer\TransactionCollection::class,
                    parameters: ['startIndex' => TypeFactory::getInteger(), 'count' => TypeFactory::getInteger(), 'search' => TypeFactory::getString()],
                    throws: [401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'transaction.get' => new Operation(
                    action: Consumer\Action\Transaction\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/transaction/$transaction_id<[0-9]+|^~>',
                    httpCode: 200,
                    outgoing: Model\Consumer\Transaction::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'account.get' => new Operation(
                    action: Consumer\Action\User\Get::class,
                    httpMethod: 'GET',
                    httpPath: '/account',
                    httpCode: 200,
                    outgoing: Model\Consumer\UserAccount::class,
                    throws: [401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'account.update' => new Operation(
                    action: Consumer\Action\User\Update::class,
                    httpMethod: 'PUT',
                    httpPath: '/account',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Consumer\UserAccount::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'account.changePassword' => new Operation(
                    action: Consumer\Action\User\ChangePassword::class,
                    httpMethod: 'PUT',
                    httpPath: '/account/change_password',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Backend\AccountChangePassword::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'account.activate' => new Operation(
                    action: Consumer\Action\User\Activate::class,
                    httpMethod: 'POST',
                    httpPath: '/activate',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Consumer\UserActivate::class,
                    throws: [400 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    public: true,
                ),
                'account.getApp' => new Operation(
                    action: Consumer\Action\User\GetApp::class,
                    httpMethod: 'GET',
                    httpPath: '/authorize',
                    httpCode: 200,
                    outgoing: Model\Consumer\AuthorizeMeta::class,
                    throws: [401 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    public: true,
                ),
                'account.authorize' => new Operation(
                    action: Consumer\Action\User\Authorize::class,
                    httpMethod: 'POST',
                    httpPath: '/authorize',
                    httpCode: 200,
                    outgoing: Model\Consumer\AuthorizeResponse::class,
                    incoming: Model\Consumer\AuthorizeRequest::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    public: true,
                ),
                'account.login' => new Operation(
                    action: Consumer\Action\User\Login::class,
                    httpMethod: 'POST',
                    httpPath: '/login',
                    httpCode: 200,
                    outgoing: Model\Consumer\UserJWT::class,
                    incoming: Model\Consumer\UserLogin::class,
                    throws: [400 => Model\Common\Message::class, 401 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    public: true,
                ),
                'account.refresh' => new Operation(
                    action: Consumer\Action\User\Refresh::class,
                    httpMethod: 'PUT',
                    httpPath: '/login',
                    httpCode: 200,
                    outgoing: Model\Consumer\UserJWT::class,
                    incoming: Model\Consumer\UserRefresh::class,
                    throws: [400 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    public: true,
                ),
                'account.register' => new Operation(
                    action: Consumer\Action\User\Register::class,
                    httpMethod: 'POST',
                    httpPath: '/register',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Consumer\UserRegister::class,
                    throws: [400 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    public: true,
                ),
                'account.requestPasswordReset' => new Operation(
                    action: Consumer\Action\User\ResetPassword\Request::class,
                    httpMethod: 'POST',
                    httpPath: '/password_reset',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Consumer\UserEmail::class,
                    throws: [400 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    public: true,
                ),
                'account.executePasswordReset' => new Operation(
                    action: Consumer\Action\User\ResetPassword\Execute::class,
                    httpMethod: 'PUT',
                    httpPath: '/password_reset',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    incoming: Model\Consumer\UserPasswordReset::class,
                    throws: [400 => Model\Common\Message::class, 404 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    public: true,
                ),
                'identity.getAll' => new Operation(
                    action: Consumer\Action\Identity\GetAll::class,
                    httpMethod: 'GET',
                    httpPath: '/identity',
                    httpCode: 200,
                    outgoing: Model\Consumer\IdentityCollection::class,
                    parameters: ['appId' => TypeFactory::getInteger()->setDeprecated(true), 'appKey' => TypeFactory::getString()],
                    throws: [400 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    public: true,
                ),
                'identity.exchange' => new Operation(
                    action: Consumer\Action\Identity\Exchange::class,
                    httpMethod: 'GET',
                    httpPath: '/identity/:identity/exchange',
                    httpCode: 200,
                    outgoing: Passthru::class,
                    throws: [400 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    public: true,
                ),
                'identity.redirect' => new Operation(
                    action: Consumer\Action\Identity\Redirect::class,
                    httpMethod: 'GET',
                    httpPath: '/identity/:identity/redirect',
                    httpCode: 200,
                    outgoing: Passthru::class,
                    throws: [400 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    public: true,
                ),
            ],
            'system' => [
                'connection.callback' => new Operation(
                    action: System\Action\Connection\Callback::class,
                    httpMethod: 'GET',
                    httpPath: '/connection/:name/callback',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    public: true,
                ),
                'meta.getAbout' => new Operation(
                    action: System\Action\Meta\GetAbout::class,
                    httpMethod: 'GET',
                    httpPath: '/about',
                    httpCode: 200,
                    outgoing: Model\System\About::class,
                    public: true,
                ),
                'meta.getDebug' => new Operation(
                    action: System\Action\Meta\GetDebug::class,
                    httpMethod: 'POST',
                    httpPath: '/debug',
                    httpCode: 200,
                    outgoing: Passthru::class,
                    incoming: Passthru::class,
                    public: true,
                ),
                'meta.getHealth' => new Operation(
                    action: System\Action\Meta\GetHealth::class,
                    httpMethod: 'GET',
                    httpPath: '/health',
                    httpCode: 200,
                    outgoing: Model\System\HealthCheck::class,
                    public: true,
                ),
                'meta.getOAuth2Configuration' => new Operation(
                    action: System\Action\Meta\GetOAuth2Configuration::class,
                    httpMethod: 'GET',
                    httpPath: '/oauth-authorization-server',
                    httpCode: 200,
                    outgoing: Model\System\OAuthConfiguration::class,
                    public: true,
                ),
                'meta.getRoutes' => new Operation(
                    action: System\Action\Meta\GetRoutes::class,
                    httpMethod: 'GET',
                    httpPath: '/route',
                    httpCode: 200,
                    outgoing: Model\System\Route::class,
                    public: true,
                ),
                'meta.getSchema' => new Operation(
                    action: System\Action\Meta\GetSchema::class,
                    httpMethod: 'GET',
                    httpPath: '/schema/:name',
                    httpCode: 200,
                    outgoing: Model\System\Schema::class,
                    throws: [404 => Model\Common\Message::class, 410 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                    public: true,
                ),
                'payment.webhook' => new Operation(
                    action: System\Action\Payment\Webhook::class,
                    httpMethod: 'POST',
                    httpPath: '/payment/:provider/webhook',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [500 => Model\Common\Message::class],
                    public: true,
                ),
            ],
            'authorization' => [
                'revoke' => new Operation(
                    action: Authorization\Action\Revoke::class,
                    httpMethod: 'POST',
                    httpPath: '/revoke',
                    httpCode: 200,
                    outgoing: Model\Common\Message::class,
                    throws: [400 => Model\Common\Message::class, 500 => Model\Common\Message::class],
                ),
                'getWhoami' => new Operation(
                    action: Authorization\Action\GetWhoami::class,
                    httpMethod: 'GET',
                    httpPath: '/whoami',
                    httpCode: 200,
                    outgoing: Model\Backend\User::class,
                    throws: [500 => Model\Common\Message::class],
                ),
            ],
        ];
    }

    /**
     * Reads files in new line neutral way that means we always use \n
     */
    private static function readFile(string $file): string
    {
        $lines = file(__DIR__ . '/resources/' . $file);
        $lines = array_map('rtrim', $lines);
        return implode("\n", $lines);
    }

    private static function getContainer(): ContainerInterface
    {
        global $container;
        if ($container instanceof ContainerInterface) {
            return $container;
        }

        throw new \RuntimeException('Could not detect global container');
    }
}
